@@ -36,6 +36,7 @@
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/math/go-math.h>
 #include <goffice/utils/go-format.h>
+#include <goffice/utils/go-persist.h>
 #include <goffice/data/go-data-simple.h>
 
 #include <gsf/gsf-impl-utils.h>
@@ -560,7 +561,7 @@ map_log_to_view (GogAxisMap *map, double value)
 	double result;
 	
 	if (value <= 0.) 
-		/* Make libart happy */
+		/* Make libart happy, do we still need it? */
 		result = map->axis->inverted ? -DBL_MAX : DBL_MAX;
 	else
 		result = map->axis->inverted ? 
@@ -1077,7 +1078,8 @@ role_grid_line_major_can_add (GogObject const *parent)
 	GogAxis *axis = GOG_AXIS (parent);
 	GogAxisType type = gog_axis_get_atype (axis);
 	
-	return ((type == GOG_AXIS_X || type == GOG_AXIS_Y || type == GOG_AXIS_RADIAL || 
+	return ((type == GOG_AXIS_X || type == GOG_AXIS_Y || type == GOG_AXIS_Z
+		 || type == GOG_AXIS_RADIAL ||
 		 (type == GOG_AXIS_CIRCULAR && !gog_axis_is_discrete (axis))) &&
 		gog_axis_get_grid_line (GOG_AXIS (parent), TRUE) == NULL);
 }
@@ -1089,7 +1091,7 @@ role_grid_line_minor_can_add (GogObject const *parent)
 	GogAxisType type = gog_axis_get_atype (axis);
 	
 	return (!gog_axis_is_discrete (GOG_AXIS (parent)) &&
-		(type == GOG_AXIS_X || type == GOG_AXIS_Y || 
+		(type == GOG_AXIS_X || type == GOG_AXIS_Y || type == GOG_AXIS_Z ||
 		 type == GOG_AXIS_RADIAL || type == GOG_AXIS_CIRCULAR) &&
 		gog_axis_get_grid_line (GOG_AXIS (parent), FALSE) == NULL);
 }
@@ -1132,7 +1134,8 @@ role_label_can_add (GogObject const *parent)
 	GogAxisType type = gog_axis_get_atype (GOG_AXIS (parent));
 	
 	return (type == GOG_AXIS_X ||
-		type == GOG_AXIS_Y);
+		type == GOG_AXIS_Y ||
+		type == GOG_AXIS_Z);
 }
 
 /**
@@ -1784,18 +1787,18 @@ gog_axis_class_init (GObjectClass *gobject_klass)
 		g_param_spec_boolean ("invert-axis", _("Invert axis"),
 			_("Scale from high to low rather than low to high"),
 			FALSE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_PROP_MAP,
 		g_param_spec_string ("map-name", _("MapName"),
 			_("The name of the map for scaling"),
 			"linear", 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_PROP_ASSIGNED_FORMAT_STR_XL,
 		g_param_spec_string ("assigned-format-string-XL", 
 			_("Assigned XL format"),
 			_("The user assigned format to use for non-discrete axis labels (XL format)"),
 			"General", 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_PROP_CIRCULAR_ROTATION,
 		g_param_spec_double ("circular-rotation", 
 			_("Rotation of circular axis"), 
@@ -1803,13 +1806,13 @@ gog_axis_class_init (GObjectClass *gobject_klass)
 			GOG_AXIS_CIRCULAR_ROTATION_MIN, 
 			GOG_AXIS_CIRCULAR_ROTATION_MAX,
 			0.0, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_PROP_POLAR_UNIT,
 		g_param_spec_string ("polar-unit", 
 			_("Polar axis set unit"), 
 			_("Polar axis set unit"), 
 			polar_units[GOG_AXIS_POLAR_UNIT_DEGREES].name,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	gog_object_register_roles (gog_klass, roles, G_N_ELEMENTS (roles));
 
@@ -2224,6 +2227,57 @@ typedef GogAxisBaseViewClass	GogAxisViewClass;
 
 static GogViewClass *aview_parent_klass;
 
+static void
+gog_axis_view_padding_request_3d (GogView *view, GogView *child,
+                                  GogViewAllocation const *plot_area,
+				  GogViewPadding *label_padding)
+{
+	GogViewAllocation child_bbox;
+	GogViewAllocation label_pos;
+	GogViewAllocation tmp = *plot_area;
+	GogViewRequisition req, available;
+	GogStyle *style = gog_styled_object_get_style (GOG_STYLED_OBJECT (child->model));
+	double angle;
+
+	gog_axis_base_view_label_position_request (view, plot_area, &label_pos);
+	if (style->text_layout.auto_angle) {
+		angle = atan2 (label_pos.w, label_pos.h) * 180. / M_PI;
+		if (angle < 0.)
+			angle += 180.;
+		style->text_layout.angle = (angle > 45. && angle < 135.)? 90. : 0.;
+	}
+
+	available.w = plot_area->w;
+	available.h = plot_area->h;
+	
+	gog_view_size_request (child, &available, &req);
+	
+	if (req.w == 0 || req.h == 0)
+		return;
+
+	child_bbox.x = label_pos.x + label_pos.w;
+	if (label_pos.w < 0)
+		child_bbox.x -= req.w;
+	child_bbox.y = label_pos.y + label_pos.h;
+	if (label_pos.h < 0)
+		child_bbox.y -= req.h;
+
+	child_bbox.w = req.w;
+	child_bbox.h = req.h;
+
+	tmp.x -= label_padding->wl;
+	tmp.w += label_padding->wl + label_padding->wr;
+	tmp.y -= label_padding->hb;
+	tmp.h += label_padding->hb + label_padding->ht;
+	
+	label_padding->wl += MAX (0, tmp.x - child_bbox.x);
+	label_padding->ht += MAX (0, tmp.y - child_bbox.y);
+	label_padding->wr += MAX (0, child_bbox.x + child_bbox.w
+				  - tmp.x - tmp.w);
+	label_padding->hb += MAX (0, child_bbox.y + child_bbox.h
+				  - tmp.y - tmp.h);
+}
+
 static void
 gog_axis_view_padding_request (GogView *view, 
 			       GogViewAllocation const *bbox,
@@ -2237,6 +2291,7 @@ gog_axis_view_padding_request (GogView *view,
 	GogViewAllocation tmp = *bbox;
 	GogViewRequisition req, available;
 	GogViewPadding label_padding, child_padding;
+	GogChart *chart = GOG_CHART (gog_object_get_parent (view->model));
 	GSList *ptr;
 	double const pad_h = gog_renderer_pt2r_y (view->renderer, PAD_HACK);
 	double const pad_w = gog_renderer_pt2r_x (view->renderer, PAD_HACK);
@@ -2249,36 +2304,50 @@ gog_axis_view_padding_request (GogView *view,
 		child = ptr->data;
 		pos = child->model->position;
 		if (IS_GOG_LABEL (child->model) && !(pos & GOG_POSITION_MANUAL)) {
-			available.w = bbox->w;
-			available.h = bbox->h;
-			gog_view_size_request (child, &available, &req);
-			if (type == GOG_AXIS_X)
-				switch (axis_pos) {
-					case GOG_AXIS_AT_HIGH:
-						label_padding.ht += req.h + pad_h;
-						break;
-					case GOG_AXIS_AT_LOW:
-					default:
-						label_padding.hb += req.h + pad_h;
-						break;
-				}
-			else
-				switch (axis_pos) {
-					case GOG_AXIS_AT_HIGH:
-						label_padding.wr += req.w + pad_w;
-						break;
-					case GOG_AXIS_AT_LOW:
-					default:
-						label_padding.wl += req.w + pad_w;
-						break;
-				}
+			if (gog_chart_is_3d (chart)) {
+				gog_axis_view_padding_request_3d (view, child,
+					bbox, &label_padding);
+			} else {
+				available.w = bbox->w;
+				available.h = bbox->h;
+				gog_view_size_request (child, &available, &req);
+				if (type == GOG_AXIS_X)
+					switch (axis_pos) {
+						case GOG_AXIS_AT_HIGH:
+							label_padding.ht += req.h + pad_h;
+							break;
+						case GOG_AXIS_AT_LOW:
+						default:
+							label_padding.hb += req.h + pad_h;
+							break;
+					}
+				else
+					switch (axis_pos) {
+						case GOG_AXIS_AT_HIGH:
+							label_padding.wr += req.w + pad_w;
+							break;
+						case GOG_AXIS_AT_LOW:
+						default:
+							label_padding.wl += req.w + pad_w;
+							break;
+					}
+			}
 		}
 	}
 
-	tmp.x += label_padding.wl;
-	tmp.w -= label_padding.wl + label_padding.wr;
-	tmp.y += label_padding.hb;
-	tmp.h -= label_padding.hb + label_padding.ht;
+	if (gog_chart_is_3d (chart)) {
+		/* For 3d chart we have to calculate how much more padding
+		 * is needed for the axis itself */
+		tmp.x -= label_padding.wl;
+		tmp.w += label_padding.wl + label_padding.wr;
+		tmp.y -= label_padding.hb;
+		tmp.h += label_padding.hb + label_padding.ht;
+	} else {
+		tmp.x += label_padding.wl;
+		tmp.w -= label_padding.wl + label_padding.wr;
+		tmp.y += label_padding.hb;
+		tmp.h -= label_padding.hb + label_padding.ht;
+	}
 
 	(aview_parent_klass->padding_request) (view, &tmp, padding);
 
@@ -2299,6 +2368,30 @@ gog_axis_view_padding_request (GogView *view,
 	padding->hb += label_padding.hb;
 }
 
+static void
+gog_axis_view_size_allocate_3d (GogView *view, GogView *child,
+                                GogViewAllocation const *plot_area)
+{
+	GogViewAllocation child_bbox;
+	GogViewAllocation label_pos;
+	GogViewRequisition req, available;
+
+	gog_view_size_request (child, &available, &req);
+	gog_axis_base_view_label_position_request (view, plot_area, &label_pos);
+	
+	child_bbox.x = label_pos.x + label_pos.w;
+	if (label_pos.w < 0)
+		child_bbox.x -= req.w;
+	child_bbox.y = label_pos.y + label_pos.h;
+	if (label_pos.h < 0)
+		child_bbox.y -= req.h;
+
+	child_bbox.w = req.w;
+	child_bbox.h = req.h;
+	
+	gog_view_size_allocate (child, &child_bbox);
+}
+
 static void
 gog_axis_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 {
@@ -2312,12 +2405,10 @@ gog_axis_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 	GogViewRequisition req, available;
 	GogObjectPosition pos;
 	GogAxisPosition axis_pos;
+	GogChart *chart = GOG_CHART (gog_object_get_parent (view->model));
 	double const pad_h = gog_renderer_pt2r_y (view->renderer, PAD_HACK);
 	double const pad_w = gog_renderer_pt2r_x (view->renderer, PAD_HACK);
 
-	if (gog_chart_is_3d (GOG_CHART (gog_object_get_parent (view->model))))
-	    return;
-
 	available.w = bbox->w;
 	available.h = bbox->h;
 
@@ -2334,6 +2425,11 @@ gog_axis_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 		} else {
 			if (GOG_POSITION_IS_SPECIAL (pos)) {
 				if (IS_GOG_LABEL (child->model)) {
+					if (gog_chart_is_3d (chart)) {
+						gog_axis_view_size_allocate_3d (view,
+							child, plot_area);
+						return;
+					}
 					gog_view_size_request (child, &available, &req);
 					if (type == GOG_AXIS_X) {
 						child_bbox.x = plot_area->x +
