@@ -36,9 +36,6 @@
 
 #include <glib/gi18n-lib.h>
 
-#include <libart_lgpl/art_render_gradient.h>
-#include <libart_lgpl/art_render_svp.h>
-#include <libart_lgpl/art_render_mask.h>
 #include <pango/pangocairo.h>
 
 #include <cairo.h>
@@ -113,86 +110,6 @@ typedef struct {
 
 #define GRC_LINEAR_SCALE(w,scale) (go_sub_epsilon (w) <= 0.0 ? GOG_RENDERER_HAIRLINE_WIDTH_PTS : w) * scale
 
-static void
-grc_path_raw (cairo_t *cr, ArtVpath *vpath, ArtBpath *bpath)
-{
-	if (vpath) {
-		while (vpath->code != ART_END) {
-			switch (vpath->code) {
-				case ART_MOVETO_OPEN:
-				case ART_MOVETO:
-					cairo_move_to (cr, GO_CAIRO_CLAMP (vpath->x),
-						       GO_CAIRO_CLAMP (vpath->y));
-					break;
-				case ART_LINETO:
-					cairo_line_to (cr, GO_CAIRO_CLAMP (vpath->x),
-						       GO_CAIRO_CLAMP (vpath->y));
-					break;
-				default:
-					break;
-			}
-			vpath++;
-		}
-	} else
-		while (bpath->code != ART_END) {
-			switch (bpath->code) {
-				case ART_MOVETO_OPEN:
-				case ART_MOVETO:
-					cairo_move_to (cr, GO_CAIRO_CLAMP (bpath->x3),
-						       GO_CAIRO_CLAMP (bpath->y3));
-					break;
-				case ART_LINETO:
-					cairo_line_to (cr, GO_CAIRO_CLAMP (bpath->x3),
-						       GO_CAIRO_CLAMP (bpath->y3));
-					break;
-				case ART_CURVETO:
-					cairo_curve_to (cr, GO_CAIRO_CLAMP (bpath->x1),
-							GO_CAIRO_CLAMP (bpath->y1),
-							GO_CAIRO_CLAMP (bpath->x2),
-							GO_CAIRO_CLAMP (bpath->y2),
-							GO_CAIRO_CLAMP (bpath->x3),
-							GO_CAIRO_CLAMP (bpath->y3));
-					break;
-				default:
-					break;
-			}
-			bpath++;
-		}
-}
-
-static void
-grc_path_sharp (cairo_t *cr, ArtVpath *vpath, ArtBpath *bpath, double line_width)
-{
-	gboolean even = ((int) (go_fake_ceil (line_width)) % 2 == 0) && line_width > 1.0;
-
-	if (vpath)
-		while (vpath->code != ART_END) {
-			switch (vpath->code) {
-				case ART_MOVETO_OPEN:
-				case ART_MOVETO:
-					cairo_move_to (cr, GO_CAIRO_CLAMP_SNAP (vpath->x, even),
-						           GO_CAIRO_CLAMP_SNAP (vpath->y, even));
-					break;
-				case ART_LINETO:
-					cairo_line_to (cr, GO_CAIRO_CLAMP_SNAP (vpath->x, even),
-						           GO_CAIRO_CLAMP_SNAP (vpath->y, even));
-					break;
-				default:
-					break;
-			}
-			vpath++;
-		}
-}
-
-static void
-grc_path (cairo_t *cr, ArtVpath *vpath, ArtBpath *bpath, double line_width, gboolean sharp)
-{
-	if (sharp)
-		grc_path_sharp (cr, vpath, bpath, line_width);
-	else
-		grc_path_raw (cr, vpath, bpath);
-}
-
 static double
 _grc_line_size (GogRenderer const *rend, double width, gboolean sharp)
 {
@@ -680,153 +597,52 @@ gog_renderer_pop_clip (GogRenderer *rend)
 	cairo_restore (rend->cairo);
 }
 
-/*****************************************************************************/
-
 static void
-grc_draw_path (GogRenderer *rend, ArtVpath const *vpath, ArtBpath const*bpath, gboolean sharp)
-{
-	GogStyle const *style = rend->cur_style;
-
-	if (gog_style_is_line_visible (style)) {
-		gboolean legend_line;
-		double width;
-
-		/* KLUDGE snap coordinate of legend line sample. */
-		legend_line =!sharp
-			&& !rend->is_vector
-			&& vpath != NULL
-			&& vpath[0].code == ART_MOVETO
-			&& vpath[1].code == ART_LINETO
-			&& vpath[2].code == ART_END
-			&& (vpath[0].x == vpath[1].x || vpath[0].y == vpath[1].y);
-
-		width = _grc_line_size (rend, style->line.width, sharp);
-
-		grc_path (rend->cairo, (ArtVpath *) vpath, (ArtBpath *) bpath, width,
-			  (sharp || legend_line) && !rend->is_vector);
-		emit_line (rend, FALSE, sharp ? GO_PATH_OPTIONS_SHARP : 0);
-	}
-
-}
-
-static void
-grc_draw_polygon (GogRenderer *rend, ArtVpath const *vpath, ArtBpath const *bpath,
-		  gboolean sharp, gboolean narrow)
+_draw_circle (GogRenderer *rend, double x, double y, double r, gboolean fill, gboolean stroke)
 {
 	GogStyle const *style;
-	double width;
-	gboolean is_outline_visible, is_fill_visible;
+	GOPath *path;
+	gboolean narrow = r < 1.5;
+	double o, o_2;
+
+	g_return_if_fail (IS_GOG_RENDERER (rend));
+	g_return_if_fail (IS_GOG_STYLE (rend->cur_style));
 
 	style = rend->cur_style;
+	narrow |= !gog_style_is_outline_visible (style);
 
-	is_outline_visible = gog_style_is_outline_visible (style) && !narrow;
-	is_fill_visible = gog_style_is_fill_visible (style);
+	path = go_path_new ();
+	go_path_set_options (path, GO_PATH_OPTIONS_SHARP);
 
-	if (!is_outline_visible && !is_fill_visible)
-		return;
+	if (!narrow) {
+		o = gog_renderer_line_size (rend, style->outline.width);
+		o_2 = o / 2.;
+	} else
+		o = o_2 = 0.;
 
-        width = _grc_line_size (rend, style->outline.width, sharp);
+	go_path_arc (path, x, y, r, r, 0, 2 * M_PI);
 
-	grc_path (rend->cairo, (ArtVpath *) vpath, (ArtBpath *) bpath, width,
-		  sharp && !rend->is_vector);
+	_draw_shape (rend, path, fill, stroke && !narrow);
 
-	if (is_fill_visible)
-		emit_fill (rend, is_outline_visible);
-
-	if (is_outline_visible)
-		emit_outline (rend, FALSE, sharp ? GO_PATH_OPTIONS_SHARP : 0);
+	go_path_free (path);
 }
 
-/**
- * gog_renderer_draw_sharp_path :
- * @rend : #GogRenderer
- * @path  : #ArtVpath
- *
- * Draws @path using the outline elements of the current style,
- * trying to make line with sharp edge.
- **/
-
 void
-gog_renderer_draw_sharp_path (GogRenderer *rend, ArtVpath *path)
+gog_renderer_draw_circle (GogRenderer *rend, double x, double y, double r)
 {
-	g_return_if_fail (IS_GOG_RENDERER (rend));
-	g_return_if_fail (rend->cur_style != NULL);
-
-	grc_draw_path (rend, path, NULL, TRUE);
+	_draw_circle (rend, x, y, r, TRUE, TRUE);
 }
 
-/**
- * gog_renderer_draw_path :
- * @rend: #GogRenderer
- * @path: #ArtVpath
- *
- * Draws @path using the outline elements of the current style.
- **/
-
 void
-gog_renderer_draw_path (GogRenderer *rend, ArtVpath const *path)
+gog_renderer_stroke_circle (GogRenderer *rend, double x, double y, double r)
 {
-	g_return_if_fail (IS_GOG_RENDERER (rend));
-	g_return_if_fail (rend->cur_style != NULL);
-
-	grc_draw_path (rend, path, NULL, FALSE);
+	_draw_circle (rend, x, y, r, FALSE, TRUE);
 }
 
-/**
- * gog_renderer_draw_sharp_polygon :
- * @rend : #GogRenderer
- * @path  : #ArtVpath
- * @narrow : if TRUE skip any outline the current style specifies.
- *
- * Draws @path and fills it with the fill elements of the current style,
- * trying to draw line with sharp edge.
- * If @narrow is false it alos outlines it using the outline elements.
- **/
-
 void
-gog_renderer_draw_sharp_polygon (GogRenderer *rend, ArtVpath *path, gboolean narrow)
+gog_renderer_fill_circle (GogRenderer *rend, double x, double y, double r)
 {
-	g_return_if_fail (IS_GOG_RENDERER (rend));
-	g_return_if_fail (rend->cur_style != NULL);
-
-	grc_draw_polygon (rend, path, NULL, TRUE, narrow);
-}
-
-/**
- * gog_renderer_draw_polygon :
- * @rend : #GogRenderer
- * @path  : #ArtVpath
- * @narrow : if TRUE skip any outline the current style specifies.
- *
- * Draws @path and fills it with the fill elements of the current style.
- * If @narrow is false it alos outlines it using the outline elements.
- **/
-
-void
-gog_renderer_draw_polygon (GogRenderer *rend, ArtVpath const *path, gboolean narrow)
-{
-	g_return_if_fail (IS_GOG_RENDERER (rend));
-	g_return_if_fail (rend->cur_style != NULL);
-
-	grc_draw_polygon (rend, path, NULL, FALSE, narrow);
-}
-
-
-/**
- * gog_renderer_draw_bezier_path :
- * @rend : #GogRenderer
- * @path  : #ArtBpath
- *
- * Draws @path using the outline elements of the current style.
- **/
-
-void
-gog_renderer_draw_bezier_path (GogRenderer *rend, ArtBpath const *path)
-{
-	g_return_if_fail (IS_GOG_RENDERER (rend));
-	g_return_if_fail (rend->cur_style != NULL);
-
-	grc_draw_path (rend, NULL, path, FALSE);
+	_draw_circle (rend, x, y, r, TRUE, FALSE);
 }
 
 /**
@@ -933,11 +749,11 @@ gog_renderer_draw_selection_rectangle (GogRenderer *renderer, GogViewAllocation
 		GogStyle *style;
 
 		style = gog_style_new ();
-		style->line.dash_type = GO_LINE_DOT;
-		style->line.width = 0.0;
-		style->line.color = 0x0000ffB0;
+		style->outline.dash_type = GO_LINE_DOT;
+		style->outline.width = 0.0;
+		style->outline.color = 0x0000ffB0;
 		style->fill.type = GOG_FILL_STYLE_NONE;
-		style->interesting_fields = GOG_STYLE_LINE;
+		style->interesting_fields = GOG_STYLE_OUTLINE;
 
 		renderer->selection_style = style;
 	}
@@ -1045,8 +861,8 @@ gog_renderer_draw_text (GogRenderer *rend, char const *text,
 	pango_layout_set_font_description (layout, style->font.font->desc);
 	pango_layout_get_size (layout, &iw, &ih);
 
-	obr.w = rend->scale * (iw / (double)PANGO_SCALE);
-	obr.h = rend->scale * (ih / (double)PANGO_SCALE);
+	obr.w = rend->scale * ((double) iw + (double) PANGO_SCALE / 2.0) / (double) PANGO_SCALE;
+	obr.h = rend->scale * ((double) ih + (double) PANGO_SCALE / 2.0) /(double) PANGO_SCALE;
 	obr.alpha = -style->text_layout.angle * M_PI / 180.0;
 	obr.x = pos->x;
 	obr.y = pos->y;
@@ -1231,6 +1047,8 @@ gog_renderer_update (GogRenderer *rend, double w, double h)
 	gboolean redraw = TRUE;
 	gboolean size_changed;
 
+	if (w <= 0 || h <= 0)
+		return FALSE;
 	g_return_val_if_fail (IS_GOG_RENDERER (rend), FALSE);
 	g_return_val_if_fail (IS_GOG_VIEW (rend->view), FALSE);
 
@@ -1249,8 +1067,6 @@ gog_renderer_update (GogRenderer *rend, double w, double h)
 
 		rend->cairo_surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, rend->w, rend->h);
 	}
-	if (w == 0 || h == 0)
-		return FALSE;
 
 	view = rend->view;
 	graph = GOG_GRAPH (view->model);
@@ -1323,6 +1139,7 @@ gog_renderer_get_pixbuf (GogRenderer *rend)
 	if (rend->cairo_surface == NULL)
 		return NULL;
 
+#ifdef GOFFICE_WITH_GTK
 	if (rend->pixbuf == NULL) {
 		int width = cairo_image_surface_get_width (rend->cairo_surface);
 		int height = cairo_image_surface_get_height (rend->cairo_surface);
@@ -1333,6 +1150,7 @@ gog_renderer_get_pixbuf (GogRenderer *rend)
 							 width, height, rowstride, NULL, NULL);
 		go_cairo_convert_data_to_pixbuf (data, NULL, width, height, rowstride);
 	}
+#endif
 
 	return rend->pixbuf;
 }
@@ -1345,6 +1163,7 @@ gog_renderer_get_cairo_surface (GogRenderer *rend)
 	return rend->cairo_surface;
 }
 
+#ifdef GOFFICE_WITH_GTK
 static gboolean
 _gsf_gdk_pixbuf_save (const gchar *buf,
 		      gsize count,
@@ -1359,6 +1178,7 @@ _gsf_gdk_pixbuf_save (const gchar *buf,
 
 	return ok;
 }
+#endif
 
 static cairo_status_t
 _cairo_write_func (void *closure,
@@ -1450,6 +1270,7 @@ gog_renderer_export_image (GogRenderer *rend, GOImageFormat format,
 	gog_graph_get_size (rend->model, &width_in_pts, &height_in_pts);
 
 	switch (format) {
+		case GO_IMAGE_FORMAT_EPS:
 		case GO_IMAGE_FORMAT_PDF:
 		case GO_IMAGE_FORMAT_PS:
 		case GO_IMAGE_FORMAT_SVG:
@@ -1496,6 +1317,21 @@ gog_renderer_export_image (GogRenderer *rend, GOImageFormat format,
 #else
 					g_warning ("[GogRendererCairo::export_image] cairo SVG backend missing");
 					return FALSE;
+#endif
+				case GO_IMAGE_FORMAT_EPS:
+					rend->marker_as_surface = FALSE;
+#ifdef HAVE_CAIRO_PS_SURFACE_SET_EPS
+					surface = cairo_ps_surface_create_for_stream
+						(_cairo_write_func,
+						 output, width_in_pts, height_in_pts);
+					cairo_ps_surface_set_eps (surface, TRUE);
+#ifdef HAVE_CAIRO_SURFACE_SET_FALLBACK_RESOLUTION
+					cairo_surface_set_fallback_resolution (surface, x_dpi, y_dpi);
+#endif
+					break;
+#else
+					g_warning ("[GogRendererCairo::export_image] cairo EPS backend missing");
+					return FALSE;
 #endif
 				default:
 					break;
@@ -1524,13 +1360,15 @@ gog_renderer_export_image (GogRenderer *rend, GOImageFormat format,
 			if (pixbuf == NULL)
 				return FALSE;
 			format_info = go_image_get_format_info (format);
+#ifdef GOFFICE_WITH_GTK
 			if (!format_info->alpha_support)
-				output_pixbuf = gdk_pixbuf_composite_color_simple (pixbuf,
-										   gdk_pixbuf_get_width (pixbuf),
-										   gdk_pixbuf_get_height (pixbuf),
-										   GDK_INTERP_NEAREST,
-										   255, 256, 0xffffffff,
-										   0xffffffff);
+				output_pixbuf = gdk_pixbuf_composite_color_simple
+					(pixbuf,
+					 gdk_pixbuf_get_width (pixbuf),
+					 gdk_pixbuf_get_height (pixbuf),
+					 GDK_INTERP_NEAREST,
+					 255, 256, 0xffffffff,
+					 0xffffffff);
 			else
 				output_pixbuf = pixbuf;
 			result = gdk_pixbuf_save_to_callback (output_pixbuf,
@@ -1540,7 +1378,7 @@ gog_renderer_export_image (GogRenderer *rend, GOImageFormat format,
 			if (!format_info->alpha_support)
 				g_object_unref (output_pixbuf);
 			return result;
-			break;
+#endif
 	}
 
 	return FALSE;
